@@ -72,7 +72,7 @@ def update_neighbor_enabled(neigh_ip_address, enabled):
                        req_args=[neighbors.IP_ADDRESS, neighbors.CHANGES])
 def update_neighbor(neigh_ip_address, changes):
     rets = []
-    for k, v in changes.iteritems():
+    for k, v in changes.items():
         if k == neighbors.MULTI_EXIT_DISC:
             rets.append(_update_med(neigh_ip_address, v))
 
@@ -88,7 +88,7 @@ def update_neighbor(neigh_ip_address, changes):
 def _update_med(neigh_ip_address, value):
     neigh_conf = _get_neighbor_conf(neigh_ip_address)
     neigh_conf.multi_exit_disc = value
-    LOG.info('MED value for neigh: %s updated to %s' % (neigh_conf, value))
+    LOG.info('MED value for neigh: %s updated to %s', neigh_conf, value)
     return True
 
 
@@ -152,8 +152,8 @@ def get_neighbor_out_filter(neigh_ip_address):
 
 @RegisterWithArgChecks(name='neighbor.out_filter.set',
                        req_args=[neighbors.IP_ADDRESS, neighbors.OUT_FILTER])
-def set_neighbor_in_filter(neigh_ip_address, filters):
-    """Returns a neighbor in_filter for given ip address if exists."""
+def set_neighbor_out_filter(neigh_ip_address, filters):
+    """Sets the out_filter of a neighbor."""
     core = CORE_MANAGER.get_core_service()
     peer = core.peer_manager.get_by_addr(neigh_ip_address)
     peer.out_filters = filters
@@ -272,14 +272,14 @@ def get_vrfs_conf():
 @register(name='network.add')
 def add_network(prefix, next_hop=None):
     tm = CORE_MANAGER.get_core_service().table_manager
-    tm.add_to_global_table(prefix, next_hop)
+    tm.update_global_table(prefix, next_hop)
     return True
 
 
 @register(name='network.del')
 def del_network(prefix):
     tm = CORE_MANAGER.get_core_service().table_manager
-    tm.add_to_global_table(prefix, is_withdraw=True)
+    tm.update_global_table(prefix, is_withdraw=True)
     return True
 
 # =============================================================================
