@@ -228,11 +228,13 @@ class CoreService(Factory, Activity):
         server_addr = (CORE_IP, self._common_config.bgp_server_port)
         waiter = kwargs.pop('waiter')
         waiter.set()
-        server_thread, sockets = self._listen_tcp(server_addr,
-                                                  self.start_protocol)
-        self.listen_sockets = sockets
-
-        server_thread.wait()
+        if self._common_config.bgp_server_port != 0:
+            server_thread, sockets = self._listen_tcp(server_addr,
+                                                      self.start_protocol)
+            self.listen_sockets = sockets
+            server_thread.wait()
+        else:
+            self.listen_sockets = {}
         processor_thread.wait()
 
     # ========================================================================
@@ -395,7 +397,7 @@ class CoreService(Factory, Activity):
 
     def on_peer_removed(self, peer):
         if peer._neigh_conf.password:
-            # seting zero length key means deleting the key
+            # setting zero length key means deleting the key
             self._set_password(peer._neigh_conf.ip_address, '')
 
         if peer.rtc_as != self.asn:
@@ -470,7 +472,8 @@ class CoreService(Factory, Activity):
         if (host, port) in self.bmpclients:
             bmpclient = self.bmpclients[(host, port)]
             if bmpclient.started:
-                LOG.warn("bmpclient is already running for %s:%s", host, port)
+                LOG.warning("bmpclient is already running for %s:%s",
+                            host, port)
                 return False
         bmpclient = BMPClient(self, host, port)
         self.bmpclients[(host, port)] = bmpclient
@@ -479,7 +482,7 @@ class CoreService(Factory, Activity):
 
     def stop_bmp(self, host, port):
         if (host, port) not in self.bmpclients:
-            LOG.warn("no bmpclient is running for %s:%s", host, port)
+            LOG.warning("no bmpclient is running for %s:%s", host, port)
             return False
 
         bmpclient = self.bmpclients[(host, port)]
