@@ -19,16 +19,24 @@
 
 import abc
 import logging
+import six
 
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_AS_PATH
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_EXTENDED_COMMUNITIES
+from ryu.lib.packet.bgp import BGP_ATTR_TYEP_PMSI_TUNNEL_ATTRIBUTE
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_MULTI_EXIT_DISC
 from ryu.lib.packet.bgp import BGPPathAttributeOrigin
 from ryu.lib.packet.bgp import BGPPathAttributeAsPath
 from ryu.lib.packet.bgp import BGPPathAttributeExtendedCommunities
 from ryu.lib.packet.bgp import BGPTwoOctetAsSpecificExtendedCommunity
 from ryu.lib.packet.bgp import BGPPathAttributeMultiExitDisc
+from ryu.lib.packet.bgp import BGPEncapsulationExtendedCommunity
+from ryu.lib.packet.bgp import BGPPathAttributePmsiTunnel
+from ryu.lib.packet.bgp import PmsiTunnelIdIngressReplication
+from ryu.lib.packet.bgp import RF_L2_EVPN
+from ryu.lib.packet.bgp import EvpnMacIPAdvertisementNLRI
+from ryu.lib.packet.bgp import EvpnIpPrefixNLRI
 
 from ryu.services.protocols.bgp.base import OrderedDict
 from ryu.services.protocols.bgp.constants import VPN_TABLE
@@ -44,12 +52,12 @@ from ryu.services.protocols.bgp.utils.stats import RESOURCE_NAME
 LOG = logging.getLogger('bgpspeaker.info_base.vrf')
 
 
+@six.add_metaclass(abc.ABCMeta)
 class VrfTable(Table):
     """Virtual Routing and Forwarding information base.
      Keeps destination imported to given vrf in represents.
      """
 
-    __metaclass__ = abc.ABCMeta
     ROUTE_FAMILY = None
     VPN_ROUTE_FAMILY = None
     NLRI_CLASS = None
@@ -64,7 +72,7 @@ class VrfTable(Table):
 
     def init_import_maps(self, import_maps):
         LOG.debug(
-            "Initializing import maps (%s) for %s" % (import_maps, repr(self))
+            "Initializing import maps (%s) for %r", import_maps, self
         )
         del self._import_maps[:]
         importmap_manager = self._core_service.importmap_manager
@@ -86,7 +94,10 @@ class VrfTable(Table):
         """Return a key that will uniquely identify this NLRI inside
         this table.
         """
-        return str(nlri)
+        # Note: We use `prefix` representation of the NLRI, because
+        # BGP route can be identified without the route distinguisher
+        # value in the VRF space.
+        return nlri.prefix
 
     def _create_dest(self, nlri):
         return self.VRF_DEST_CLASS(self, nlri)
@@ -102,10 +113,10 @@ class VrfTable(Table):
 
         remote_route_count = 0
         local_route_count = 0
-        for dest in self.itervalues():
+        for dest in self.values():
             for path in dest.known_path_list:
-                if (hasattr(path.source, 'version_num')
-                        or path.source == VPN_TABLE):
+                if (hasattr(path.source, 'version_num') or
+                        path.source == VPN_TABLE):
                     remote_route_count += 1
                 else:
                     local_route_count += 1
@@ -115,7 +126,7 @@ class VrfTable(Table):
                 LOCAL_ROUTES: local_route_count}
 
     def import_vpn_paths_from_table(self, vpn_table, import_rts=None):
-        for vpn_dest in vpn_table.itervalues():
+        for vpn_dest in vpn_table.values():
             vpn_path = vpn_dest.best_path
             if not vpn_path:
                 continue
@@ -133,7 +144,8 @@ class VrfTable(Table):
                 self.import_vpn_path(vpn_path)
 
     def import_vpn_path(self, vpn_path):
-        """Imports `vpnv(4|6)_path` into `vrf(4|6)_table`.
+        """Imports `vpnv(4|6)_path` into `vrf(4|6)_table` or `evpn_path`
+        into vrfevpn_table`.
 
         :Parameters:
             - `vpn_path`: (Path) VPN path that will be cloned and imported
@@ -147,21 +159,27 @@ class VrfTable(Table):
         source = vpn_path.source
         if not source:
             source = VRF_TABLE
-        ip, masklen = vpn_path.nlri.prefix.split('/')
-        vrf_nlri = self.NLRI_CLASS(length=int(masklen), addr=ip)
 
-        vpn_nlri = vpn_path.nlri
-        puid = self.VRF_PATH_CLASS.create_puid(vpn_nlri.route_dist,
-                                               vpn_nlri.prefix)
+        if self.VPN_ROUTE_FAMILY == RF_L2_EVPN:
+            # Because NLRI class is the same if the route family is EVPN,
+            # we re-use the NLRI instance.
+            vrf_nlri = vpn_path.nlri
+        else:  # self.VPN_ROUTE_FAMILY in [RF_IPv4_VPN, RF_IPv6_VPN]
+            # Copy NLRI instance
+            ip, masklen = vpn_path.nlri.prefix.split('/')
+            vrf_nlri = self.NLRI_CLASS(length=int(masklen), addr=ip)
+
         vrf_path = self.VRF_PATH_CLASS(
-            puid,
-            source,
-            vrf_nlri,
-            vpn_path.source_version_num,
+            puid=self.VRF_PATH_CLASS.create_puid(
+                vpn_path.nlri.route_dist,
+                vpn_path.nlri.prefix),
+            source=source,
+            nlri=vrf_nlri,
+            src_ver_num=vpn_path.source_version_num,
             pattrs=vpn_path.pathattr_map,
             nexthop=vpn_path.nexthop,
             is_withdraw=vpn_path.is_withdraw,
-            label_list=vpn_path.nlri.label_list
+            label_list=getattr(vpn_path.nlri, 'label_list', None),
         )
         if self._is_vrf_path_already_in_table(vrf_path):
             return None
@@ -187,7 +205,7 @@ class VrfTable(Table):
 
     def apply_import_maps(self):
         changed_dests = []
-        for dest in self.itervalues():
+        for dest in self.values():
             assert isinstance(dest, VrfDest)
             for import_map in self._import_maps:
                 for path in dest.known_path_list:
@@ -196,9 +214,9 @@ class VrfTable(Table):
                         changed_dests.append(dest)
         return changed_dests
 
-    def insert_vrf_path(self, ip_nlri, next_hop=None,
-                        gen_lbl=False, is_withdraw=False):
-        assert ip_nlri
+    def insert_vrf_path(self, nlri, next_hop=None,
+                        gen_lbl=False, is_withdraw=False, **kwargs):
+        assert nlri
         pattrs = None
         label_list = []
         vrf_conf = self.vrf_conf
@@ -229,6 +247,12 @@ class VrfTable(Table):
                                    local_administrator=int(local_admin),
                                    subtype=subtype))
 
+            # Set Tunnel Encapsulation Attribute
+            tunnel_type = kwargs.get('tunnel_type', None)
+            if tunnel_type:
+                communities.append(
+                    BGPEncapsulationExtendedCommunity.from_str(tunnel_type))
+
             pattrs[BGP_ATTR_TYPE_EXTENDED_COMMUNITIES] = \
                 BGPPathAttributeExtendedCommunities(communities=communities)
             if vrf_conf.multi_exit_disc:
@@ -251,11 +275,32 @@ class VrfTable(Table):
                 # If we do not have next_hop, get a new label.
                 label_list.append(table_manager.get_next_vpnv4_label())
 
+            # Set PMSI Tunnel Attribute
+            pmsi_tunnel_type = kwargs.get('pmsi_tunnel_type', None)
+            if pmsi_tunnel_type is not None:
+                from ryu.services.protocols.bgp.api.prefix import (
+                    PMSI_TYPE_INGRESS_REP)
+                if pmsi_tunnel_type == PMSI_TYPE_INGRESS_REP:
+                    tunnel_id = PmsiTunnelIdIngressReplication(
+                        tunnel_endpoint_ip=self._core_service.router_id)
+                else:  # pmsi_tunnel_type == PMSI_TYPE_NO_TUNNEL_INFO
+                    tunnel_id = None
+                pattrs[BGP_ATTR_TYEP_PMSI_TUNNEL_ATTRIBUTE] = \
+                    BGPPathAttributePmsiTunnel(pmsi_flags=0,
+                                               tunnel_type=pmsi_tunnel_type,
+                                               tunnel_id=tunnel_id)
+
+            # Set MPLS labels with the generated labels
+            if gen_lbl and isinstance(nlri, EvpnMacIPAdvertisementNLRI):
+                nlri.mpls_labels = label_list[:2]
+            elif gen_lbl and isinstance(nlri, EvpnIpPrefixNLRI):
+                nlri.mpls_label = label_list[0]
+
         puid = self.VRF_PATH_CLASS.create_puid(
-            vrf_conf.route_dist, ip_nlri.prefix
-        )
+            vrf_conf.route_dist, nlri.prefix)
+
         path = self.VRF_PATH_CLASS(
-            puid, None, ip_nlri, 0, pattrs=pattrs,
+            puid, None, nlri, 0, pattrs=pattrs,
             nexthop=next_hop, label_list=label_list,
             is_withdraw=is_withdraw
         )
@@ -273,14 +318,21 @@ class VrfTable(Table):
         return super(VrfTable, self).clean_uninteresting_paths(interested_rts)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class VrfDest(Destination):
     """Base class for VRF destination."""
-    __metaclass__ = abc.ABCMeta
 
     def __init__(self, table, nlri):
         super(VrfDest, self).__init__(table, nlri)
         self._route_dist = self._table.vrf_conf.route_dist
 
+    @property
+    def nlri_str(self):
+        # Returns `prefix` without the route distinguisher value, because
+        # a destination in VRF space can be identified without the route
+        # distinguisher.
+        return self._nlri.prefix
+
     def _best_path_lost(self):
         # Have to send update messages for withdraw of best-path to Network
         # controller or Global table.
@@ -306,7 +358,7 @@ class VrfDest(Destination):
             tm.learn_path(gpath)
 
     def _new_best_path(self, best_path):
-        LOG.debug('New best path selected for destination %s' % (self))
+        LOG.debug('New best path selected for destination %s', self)
 
         old_best_path = self._best_path
         assert (best_path != old_best_path)
@@ -335,8 +387,8 @@ class VrfDest(Destination):
             gpath = best_path.clone_to_vpn(self._route_dist)
             tm = self._core_service.table_manager
             tm.learn_path(gpath)
-            LOG.debug('VRF table %s has new best path: %s' %
-                      (self._route_dist, self.best_path))
+            LOG.debug('VRF table %s has new best path: %s',
+                      self._route_dist, self.best_path)
 
     def _remove_withdrawals(self):
         """Removes withdrawn paths.
@@ -348,7 +400,7 @@ class VrfDest(Destination):
         stopped by the same policies.
         """
 
-        LOG.debug('Removing %s withdrawals' % len(self._withdraw_list))
+        LOG.debug('Removing %s withdrawals', len(self._withdraw_list))
 
         # If we have not withdrawals, we have nothing to do.
         if not self._withdraw_list:
@@ -358,7 +410,7 @@ class VrfDest(Destination):
         # delete these withdraws.
         if not self._known_path_list:
             LOG.debug('Found %s withdrawals for path(s) that did not get'
-                      ' installed.' % len(self._withdraw_list))
+                      ' installed.', len(self._withdraw_list))
             del (self._withdraw_list[:])
             return
 
@@ -380,13 +432,13 @@ class VrfDest(Destination):
                 # We do no have any match for this withdraw.
             if not match:
                 LOG.debug('No matching path for withdraw found, may be path '
-                          'was not installed into table: %s' %
+                          'was not installed into table: %s',
                           withdraw)
             # If we have partial match.
         if len(matches) != len(self._withdraw_list):
             LOG.debug('Did not find match for some withdrawals. Number of '
-                      'matches(%s), number of withdrawals (%s)' %
-                      (len(matches), len(self._withdraw_list)))
+                      'matches(%s), number of withdrawals (%s)',
+                      len(matches), len(self._withdraw_list))
 
         # Clear matching paths and withdrawals.
         for match in matches:
@@ -409,14 +461,14 @@ class VrfDest(Destination):
                 # version num. as new_paths are implicit withdrawal of old
                 # paths and when doing RouteRefresh (not EnhancedRouteRefresh)
                 # we get same paths again.
-                if (new_path.puid == path.puid):
+                if new_path.puid == path.puid:
                     old_paths.append(path)
                     break
 
             for old_path in old_paths:
                 known_paths.remove(old_path)
                 LOG.debug('Implicit withdrawal of old path, since we have'
-                          ' learned new path from same source: %s' % old_path)
+                          ' learned new path from same source: %s', old_path)
 
     def _validate_path(self, path):
         if not path or not hasattr(path, 'label_list'):
@@ -424,11 +476,11 @@ class VrfDest(Destination):
                              'with attribute label_list got %s' % path)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class VrfPath(Path):
     """Represents a way of reaching an IP destination with a VPN.
     """
     __slots__ = ('_label_list', '_puid')
-    __metaclass__ = abc.ABCMeta
 
     ROUTE_FAMILY = None
     VPN_PATH_CLASS = None
@@ -465,6 +517,13 @@ class VrfPath(Path):
     def label_list(self):
         return self._label_list[:]
 
+    @property
+    def nlri_str(self):
+        # Returns `prefix` without the route distinguisher value, because
+        # a destination in VRF space can be identified without the route
+        # distinguisher.
+        return self._nlri.prefix
+
     @staticmethod
     def create_puid(route_dist, ip_prefix):
         assert route_dist and ip_prefix
@@ -488,22 +547,29 @@ class VrfPath(Path):
         return clone
 
     def clone_to_vpn(self, route_dist, for_withdrawal=False):
-        ip, masklen = self._nlri.prefix.split('/')
-        vpn_nlri = self.VPN_NLRI_CLASS(length=int(masklen),
-                                       addr=ip,
-                                       labels=self.label_list,
-                                       route_dist=route_dist)
+        if self.ROUTE_FAMILY == RF_L2_EVPN:
+            # Because NLRI class is the same if the route family is EVPN,
+            # we re-use the NLRI instance.
+            vpn_nlri = self._nlri
+        else:  # self.ROUTE_FAMILY in [RF_IPv4_UC, RF_IPv6_UC]
+            ip, masklen = self._nlri.prefix.split('/')
+            vpn_nlri = self.VPN_NLRI_CLASS(length=int(masklen),
+                                           addr=ip,
+                                           labels=self.label_list,
+                                           route_dist=route_dist)
 
         pathattrs = None
         if not for_withdrawal:
             pathattrs = self.pathattr_map
+
         vpnv_path = self.VPN_PATH_CLASS(
-            self.source, vpn_nlri,
-            self.source_version_num,
+            source=self.source,
+            nlri=vpn_nlri,
+            src_ver_num=self.source_version_num,
             pattrs=pathattrs,
             nexthop=self.nexthop,
-            is_withdraw=for_withdrawal
-        )
+            is_withdraw=for_withdrawal)
+
         return vpnv_path
 
     def __eq__(self, b_path):
@@ -540,7 +606,7 @@ class VrfNlriImportMap(ImportMap):
     def match(self, vrf_path):
         if vrf_path.route_family != self.VRF_PATH_CLASS.ROUTE_FAMILY:
             LOG.error(
-                "vrf_paths route_family doesn\'t match importmaps"
+                "vrf_paths route_family does not match importmaps"
                 "route_family. Applied to wrong table?")
             return False
 
