@@ -6,6 +6,7 @@ from ryu.lib.packet.bgp import RF_IPv4_UC
 from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
+from ryu.lib.packet.bgp import RF_L2_EVPN
 from ryu.lib.packet.bgp import RF_RTC_UC
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_ORIGIN
 from ryu.lib.packet.bgp import BGP_ATTR_TYPE_AS_PATH
@@ -47,7 +48,7 @@ class InternalApi(object):
         vrf_name = vrf_name.encode('ascii', 'ignore')
 
         route_count = \
-            len([d for d in vrf.itervalues() if d.best_path])
+            len([d for d in vrf.values() if d.best_path])
         return {str((vrf_name, vrf_rf)): route_count}
 
     def get_vrfs_conf(self):
@@ -56,7 +57,7 @@ class InternalApi(object):
     def get_all_vrf_routes(self):
         vrfs = self._get_vrf_tables()
         ret = {}
-        for (vrf_id, vrf_rf), table in sorted(vrfs.iteritems()):
+        for (vrf_id, vrf_rf), table in sorted(vrfs.items()):
             ret[str((vrf_id, vrf_rf))] = self._get_single_vrf_routes(table)
         return ret
 
@@ -64,10 +65,10 @@ class InternalApi(object):
         vrf = self._get_vrf_table(vrf_id, vrf_rf)
         if not vrf:
             raise WrongParamError('wrong vpn name %s' % str((vrf_id, vrf_rf)))
-        return [self._dst_to_dict(d) for d in vrf.itervalues()]
+        return [self._dst_to_dict(d) for d in vrf.values()]
 
     def _get_single_vrf_routes(self, vrf_table):
-        return [self._dst_to_dict(d) for d in vrf_table.itervalues()]
+        return [self._dst_to_dict(d) for d in vrf_table.values()]
 
     def _get_vrf_table(self, vrf_name, vrf_rf):
         return CORE_MANAGER.get_core_service()\
@@ -82,29 +83,31 @@ class InternalApi(object):
             'ipv6': RF_IPv6_UC,
             'vpnv4': RF_IPv4_VPN,
             'vpnv6': RF_IPv6_VPN,
+            'evpn': RF_L2_EVPN,
             'rtfilter': RF_RTC_UC
         }
         if addr_family not in rfs:
-            raise WrongParamError('Unknown or unsupported family')
+            raise WrongParamError('Unknown or unsupported family: %s' %
+                                  addr_family)
 
         rf = rfs.get(addr_family)
         table_manager = self.get_core_service().table_manager
         gtable = table_manager.get_global_table_by_route_family(rf)
         if gtable is not None:
             return [self._dst_to_dict(dst)
-                    for dst in sorted(gtable.itervalues())]
+                    for dst in sorted(gtable.values())]
         else:
             return []
 
     def _dst_to_dict(self, dst):
         ret = {'paths': [],
-               'prefix': dst.nlri.formatted_nlri_str}
+               'prefix': dst.nlri_str}
 
         def _path_to_dict(dst, path):
 
             path_seg_list = path.get_pattr(BGP_ATTR_TYPE_AS_PATH).path_seg_list
 
-            if type(path_seg_list) == list:
+            if isinstance(path_seg_list, list):
                 aspath = []
                 for as_path_seg in path_seg_list:
                     for as_num in as_path_seg:
@@ -140,7 +143,7 @@ class InternalApi(object):
 
             return {'best': (path == dst.best_path),
                     'bpr': bpr,
-                    'prefix': path.nlri.formatted_nlri_str,
+                    'prefix': path.nlri_str,
                     'labels': labels,
                     'nexthop': nexthop,
                     'metric': med,
@@ -154,10 +157,7 @@ class InternalApi(object):
         return ret
 
     def check_logging(self):
-        if self.log_handler and self._has_log_handler(self.log_handler):
-            return True
-        else:
-            return False
+        return self.log_handler and self._has_log_handler(self.log_handler)
 
     def check_logging_level(self):
         return logging.getLevelName(self.log_handler.level)
@@ -177,7 +177,7 @@ class InternalApi(object):
                 route_families.extend(SUPPORTED_GLOBAL_RF)
             else:
                 route_family = RouteFamily(afi, safi)
-                if (route_family not in SUPPORTED_GLOBAL_RF):
+                if route_family not in SUPPORTED_GLOBAL_RF:
                     raise WrongParamError('Not supported address-family'
                                           ' %s, %s' % (afi, safi))
                 route_families.append(route_family)
