@@ -18,16 +18,21 @@
 """
 from abc import abstractmethod
 import logging
+import numbers
+
 import netaddr
 
 from ryu.lib.packet.bgp import RF_IPv4_UC
 from ryu.lib.packet.bgp import RF_IPv6_UC
 from ryu.lib.packet.bgp import RF_IPv4_VPN
 from ryu.lib.packet.bgp import RF_IPv6_VPN
+from ryu.lib.packet.bgp import RF_L2_EVPN
 from ryu.lib.packet.bgp import RF_RTC_UC
+from ryu.lib.packet.bgp import BGPOptParamCapabilityFourOctetAsNumber
 from ryu.lib.packet.bgp import BGPOptParamCapabilityEnhancedRouteRefresh
 from ryu.lib.packet.bgp import BGPOptParamCapabilityMultiprotocol
 from ryu.lib.packet.bgp import BGPOptParamCapabilityRouteRefresh
+from ryu.lib.packet.bgp import BGP_CAP_FOUR_OCTET_AS_NUMBER
 from ryu.lib.packet.bgp import BGP_CAP_ENHANCED_ROUTE_REFRESH
 from ryu.lib.packet.bgp import BGP_CAP_MULTIPROTOCOL
 from ryu.lib.packet.bgp import BGP_CAP_ROUTE_REFRESH
@@ -37,10 +42,12 @@ from ryu.services.protocols.bgp.rtconf.base import ADVERTISE_PEER_AS
 from ryu.services.protocols.bgp.rtconf.base import BaseConf
 from ryu.services.protocols.bgp.rtconf.base import BaseConfListener
 from ryu.services.protocols.bgp.rtconf.base import CAP_ENHANCED_REFRESH
+from ryu.services.protocols.bgp.rtconf.base import CAP_FOUR_OCTET_AS_NUMBER
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV4
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_IPV6
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV4
 from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_VPNV6
+from ryu.services.protocols.bgp.rtconf.base import CAP_MBGP_EVPN
 from ryu.services.protocols.bgp.rtconf.base import CAP_REFRESH
 from ryu.services.protocols.bgp.rtconf.base import CAP_RTC
 from ryu.services.protocols.bgp.rtconf.base import compute_optional_conf
@@ -59,8 +66,7 @@ from ryu.services.protocols.bgp.rtconf.base import SITE_OF_ORIGINS
 from ryu.services.protocols.bgp.rtconf.base import validate
 from ryu.services.protocols.bgp.rtconf.base import validate_med
 from ryu.services.protocols.bgp.rtconf.base import validate_soo_list
-from ryu.services.protocols.bgp.utils.validation import is_valid_ipv4
-from ryu.services.protocols.bgp.utils.validation import is_valid_old_asn
+from ryu.services.protocols.bgp.utils.validation import is_valid_asn
 from ryu.services.protocols.bgp.info_base.base import Filter
 from ryu.services.protocols.bgp.info_base.base import PrefixFilter
 from ryu.services.protocols.bgp.info_base.base import AttributeMap
@@ -74,6 +80,7 @@ ENABLED = 'enabled'
 CHANGES = 'changes'
 LOCAL_ADDRESS = 'local_address'
 LOCAL_PORT = 'local_port'
+LOCAL_AS = 'local_as'
 PEER_NEXT_HOP = 'peer_next_hop'
 PASSWORD = 'password'
 IN_FILTER = 'in_filter'
@@ -91,10 +98,12 @@ CONNECT_MODE_BOTH = 'both'
 DEFAULT_CAP_GR_NULL = True
 DEFAULT_CAP_REFRESH = True
 DEFAULT_CAP_ENHANCED_REFRESH = False
+DEFAULT_CAP_FOUR_OCTET_AS_NUMBER = True
 DEFAULT_CAP_MBGP_IPV4 = True
 DEFAULT_CAP_MBGP_IPV6 = False
 DEFAULT_CAP_MBGP_VPNV4 = False
 DEFAULT_CAP_MBGP_VPNV6 = False
+DEFAULT_CAP_MBGP_EVPN = False
 DEFAULT_HOLD_TIME = 40
 DEFAULT_ENABLED = True
 DEFAULT_CAP_RTC = False
@@ -120,7 +129,7 @@ def validate_enabled(enabled):
 
 @validate(name=CHANGES)
 def validate_changes(changes):
-    for k, v in changes.iteritems():
+    for k, v in changes.items():
         if k not in (MULTI_EXIT_DISC, ENABLED, CONNECT_MODE):
             raise ConfigValueError(desc="Unknown field to change: %s" % k)
 
@@ -170,7 +179,7 @@ def validate_password(password):
 
 @validate(name=LOCAL_PORT)
 def validate_local_port(port):
-    if not isinstance(port, (int, long)):
+    if not isinstance(port, numbers.Integral):
         raise ConfigTypeError(desc='Invalid local port: %s' % port)
     if port < 1025 or port > 65535:
         raise ConfigValueError(desc='Invalid local port value: %s, has to be'
@@ -180,7 +189,7 @@ def validate_local_port(port):
 
 @validate(name=REMOTE_AS)
 def validate_remote_as(asn):
-    if not is_valid_old_asn(asn):
+    if not is_valid_asn(asn):
         raise ConfigValueError(desc='Invalid remote as value %s' % asn)
     return asn
 
@@ -218,7 +227,7 @@ def valid_filter(filter_):
         raise ConfigTypeError(desc='Invalid filter type: %s, supported filter'
                               ' types are %s'
                               % (filter_['type'],
-                                 SUPPORTED_FILTER_VALIDATORS.keys()))
+                                 list(SUPPORTED_FILTER_VALIDATORS.keys())))
 
     return SUPPORTED_FILTER_VALIDATORS[filter_['type']](filter_)
 
@@ -294,12 +303,13 @@ class NeighborConf(ConfWithId, ConfWithStats):
     REQUIRED_SETTINGS = frozenset([REMOTE_AS, IP_ADDRESS])
     OPTIONAL_SETTINGS = frozenset([CAP_REFRESH,
                                    CAP_ENHANCED_REFRESH,
+                                   CAP_FOUR_OCTET_AS_NUMBER,
                                    CAP_MBGP_IPV4, CAP_MBGP_IPV6,
                                    CAP_MBGP_VPNV4, CAP_MBGP_VPNV6,
-                                   CAP_RTC, RTC_AS, HOLD_TIME,
+                                   CAP_RTC, CAP_MBGP_EVPN, RTC_AS, HOLD_TIME,
                                    ENABLED, MULTI_EXIT_DISC, MAX_PREFIXES,
                                    ADVERTISE_PEER_AS, SITE_OF_ORIGINS,
-                                   LOCAL_ADDRESS, LOCAL_PORT,
+                                   LOCAL_ADDRESS, LOCAL_PORT, LOCAL_AS,
                                    PEER_NEXT_HOP, PASSWORD,
                                    IN_FILTER, OUT_FILTER,
                                    IS_ROUTE_SERVER_CLIENT, CHECK_FIRST_AS,
@@ -313,12 +323,17 @@ class NeighborConf(ConfWithId, ConfWithStats):
             CAP_REFRESH, DEFAULT_CAP_REFRESH, **kwargs)
         self._settings[CAP_ENHANCED_REFRESH] = compute_optional_conf(
             CAP_ENHANCED_REFRESH, DEFAULT_CAP_ENHANCED_REFRESH, **kwargs)
+        self._settings[CAP_FOUR_OCTET_AS_NUMBER] = compute_optional_conf(
+            CAP_FOUR_OCTET_AS_NUMBER,
+            DEFAULT_CAP_FOUR_OCTET_AS_NUMBER, **kwargs)
         self._settings[CAP_MBGP_IPV4] = compute_optional_conf(
             CAP_MBGP_IPV4, DEFAULT_CAP_MBGP_IPV4, **kwargs)
         self._settings[CAP_MBGP_IPV6] = compute_optional_conf(
             CAP_MBGP_IPV6, DEFAULT_CAP_MBGP_IPV6, **kwargs)
         self._settings[CAP_MBGP_VPNV4] = compute_optional_conf(
             CAP_MBGP_VPNV4, DEFAULT_CAP_MBGP_VPNV4, **kwargs)
+        self._settings[CAP_MBGP_EVPN] = compute_optional_conf(
+            CAP_MBGP_EVPN, DEFAULT_CAP_MBGP_EVPN, **kwargs)
         self._settings[CAP_MBGP_VPNV6] = compute_optional_conf(
             CAP_MBGP_VPNV6, DEFAULT_CAP_MBGP_VPNV6, **kwargs)
         self._settings[HOLD_TIME] = compute_optional_conf(
@@ -366,6 +381,13 @@ class NeighborConf(ConfWithId, ConfWithStats):
         self._settings[LOCAL_PORT] = compute_optional_conf(
             LOCAL_PORT, None, **kwargs)
 
+        # We use the global defined local (router) AS as the default
+        # local AS.
+        from ryu.services.protocols.bgp.core_manager import CORE_MANAGER
+        g_local_as = CORE_MANAGER.common_conf.local_as
+        self._settings[LOCAL_AS] = compute_optional_conf(
+            LOCAL_AS, g_local_as, **kwargs)
+
         self._settings[PEER_NEXT_HOP] = compute_optional_conf(
             PEER_NEXT_HOP, None, **kwargs)
 
@@ -373,14 +395,11 @@ class NeighborConf(ConfWithId, ConfWithStats):
             PASSWORD, None, **kwargs)
 
         # RTC configurations.
-        self._settings[CAP_RTC] = \
-            compute_optional_conf(CAP_RTC, DEFAULT_CAP_RTC, **kwargs)
+        self._settings[CAP_RTC] = compute_optional_conf(
+            CAP_RTC, DEFAULT_CAP_RTC, **kwargs)
         # Default RTC_AS is local (router) AS.
-        from ryu.services.protocols.bgp.core_manager import \
-            CORE_MANAGER
-        default_rt_as = CORE_MANAGER.common_conf.local_as
-        self._settings[RTC_AS] = \
-            compute_optional_conf(RTC_AS, default_rt_as, **kwargs)
+        self._settings[RTC_AS] = compute_optional_conf(
+            RTC_AS, g_local_as, **kwargs)
 
         # Since ConfWithId' default values use str(self) and repr(self), we
         # call super method after we have initialized other settings.
@@ -436,6 +455,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
     # Optional attributes with valid defaults.
     # =========================================================================
 
+    @property
+    def local_as(self):
+        return self._settings[LOCAL_AS]
+
     @property
     def hold_time(self):
         return self._settings[HOLD_TIME]
@@ -448,6 +471,17 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def cap_enhanced_refresh(self):
         return self._settings[CAP_ENHANCED_REFRESH]
 
+    @property
+    def cap_four_octet_as_number(self):
+        return self._settings[CAP_FOUR_OCTET_AS_NUMBER]
+
+    @cap_four_octet_as_number.setter
+    def cap_four_octet_as_number(self, cap):
+        kwargs = {CAP_FOUR_OCTET_AS_NUMBER: cap}
+        self._settings[CAP_FOUR_OCTET_AS_NUMBER] = compute_optional_conf(
+            CAP_FOUR_OCTET_AS_NUMBER,
+            DEFAULT_CAP_FOUR_OCTET_AS_NUMBER, **kwargs)
+
     @property
     def cap_mbgp_ipv4(self):
         return self._settings[CAP_MBGP_IPV4]
@@ -464,6 +498,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
     def cap_mbgp_vpnv6(self):
         return self._settings[CAP_MBGP_VPNV6]
 
+    @property
+    def cap_mbgp_evpn(self):
+        return self._settings[CAP_MBGP_EVPN]
+
     @property
     def cap_rtc(self):
         return self._settings[CAP_RTC]
@@ -549,7 +587,7 @@ class NeighborConf(ConfWithId, ConfWithStats):
 
         return does_exceed
 
-    def get_configured_capabilites(self):
+    def get_configured_capabilities(self):
         """Returns configured capabilities."""
 
         capabilities = OrderedDict()
@@ -579,6 +617,11 @@ class NeighborConf(ConfWithId, ConfWithStats):
                 BGPOptParamCapabilityMultiprotocol(
                     RF_RTC_UC.afi, RF_RTC_UC.safi))
 
+        if self.cap_mbgp_evpn:
+            mbgp_caps.append(
+                BGPOptParamCapabilityMultiprotocol(
+                    RF_L2_EVPN.afi, RF_L2_EVPN.safi))
+
         if mbgp_caps:
             capabilities[BGP_CAP_MULTIPROTOCOL] = mbgp_caps
 
@@ -590,6 +633,10 @@ class NeighborConf(ConfWithId, ConfWithStats):
             capabilities[BGP_CAP_ENHANCED_ROUTE_REFRESH] = [
                 BGPOptParamCapabilityEnhancedRouteRefresh()]
 
+        if self.cap_four_octet_as_number:
+            capabilities[BGP_CAP_FOUR_OCTET_AS_NUMBER] = [
+                BGPOptParamCapabilityFourOctetAsNumber(self.local_as)]
+
         return capabilities
 
     def __repr__(self):
@@ -599,7 +646,7 @@ class NeighborConf(ConfWithId, ConfWithStats):
                                      self.enabled)
 
     def __str__(self):
-        return 'Neighbor: %s' % (self.ip_address)
+        return 'Neighbor: %s' % self.ip_address
 
 
 class NeighborsConf(BaseConf):
@@ -626,7 +673,7 @@ class NeighborsConf(BaseConf):
         """Returns current RTC AS configured for current neighbors.
         """
         rtc_as_set = set()
-        for neigh in self._neighbors.itervalues():
+        for neigh in self._neighbors.values():
             rtc_as_set.add(neigh.rtc_as)
         return rtc_as_set
 
@@ -669,7 +716,7 @@ class NeighborsConf(BaseConf):
     @property
     def settings(self):
         return [neighbor.settings for _, neighbor in
-                self._neighbors.iteritems()]
+                self._neighbors.items()]
 
 
 class NeighborConfListener(ConfWithIdListener, ConfWithStatsListener):
