@@ -26,10 +26,24 @@ from ryu.controller import handler
 
 opts = (cfg.StrOpt('address', default='0.0.0.0', help='OVSDB address'),
         cfg.IntOpt('port', default=6640, help='OVSDB port'),
+        cfg.IntOpt('probe-interval', help='OVSDB reconnect probe interval'),
+        cfg.IntOpt('min-backoff',
+                   help=('OVSDB reconnect minimum milliseconds between '
+                         'connection attemps')),
+        cfg.IntOpt('max-backoff',
+                   help=('OVSDB reconnect maximum milliseconds between '
+                         'connection attemps')),
         cfg.StrOpt('mngr-privkey', default=None, help='manager private key'),
         cfg.StrOpt('mngr-cert', default=None, help='manager certificate'),
         cfg.ListOpt('whitelist', default=[],
-                    help='Whitelist of address to allow to connect'))
+                    help='Whitelist of address to allow to connect'),
+        cfg.ListOpt('schema-tables', default=[],
+                    help='Tables in the OVSDB schema to configure'),
+        cfg.ListOpt('schema-exclude-columns', default=[],
+                    help='Table columns in the OVSDB schema to filter out.  '
+                         'Values should be in the format: <table>.<column>.'
+                         'Ex: Bridge.netflow,Interface.statistics')
+        )
 
 cfg.CONF.register_opts(opts, 'ovsdb')
 
@@ -43,6 +57,9 @@ class OVSDB(app_manager.RyuApp):
         super(OVSDB, self).__init__(*args, **kwargs)
         self._address = self.CONF.ovsdb.address
         self._port = self.CONF.ovsdb.port
+        self._probe_interval = self.CONF.ovsdb.probe_interval
+        self._min_backoff = self.CONF.ovsdb.min_backoff
+        self._max_backoff = self.CONF.ovsdb.max_backoff
         self._clients = {}
 
     def _accept(self, server):
@@ -59,9 +76,15 @@ class OVSDB(app_manager.RyuApp):
             def check(address):
                 return True
 
-        while True:
-            # TODO(jkoelker) SSL Certificate Fingerprint check
-            sock, client_address = server.accept()
+        while self.is_active:
+            try:
+                # TODO(jkoelker) SSL Certificate Fingerprint check
+                sock, client_address = server.accept()
+
+            except:
+                if self.is_active:
+                    self.logger.exception('Error accepting connection')
+                    continue
 
             if not check(client_address[0]):
                 sock.shutdown(socket.SHUT_RDWR)
@@ -72,6 +95,26 @@ class OVSDB(app_manager.RyuApp):
             t = hub.spawn(self._start_remote, sock, client_address)
             self.threads.append(t)
 
+    def _bulk_read_handler(self, ev):
+        results = []
+
+        def done(gt, *args, **kwargs):
+            if gt in self.threads:
+                self.threads.remove(gt)
+
+            results.append(gt.wait())
+
+        threads = []
+        for c in self._clients.values():
+            gt = hub.spawn(c.read_request_handler, ev, bulk=True)
+            threads.append(gt)
+            self.threads.append(gt)
+            gt.link(done)
+
+        hub.joinall(threads)
+        rep = event.EventReadReply(None, results)
+        self.reply_to_request(ev, rep)
+
     def _proxy_event(self, ev):
         system_id = ev.system_id
         client_name = client.RemoteOvsdb.instance_name(system_id)
@@ -83,7 +126,22 @@ class OVSDB(app_manager.RyuApp):
         return self.send_event(client_name, ev)
 
     def _start_remote(self, sock, client_address):
-        app = client.RemoteOvsdb.factory(sock, client_address)
+        schema_tables = cfg.CONF.ovsdb.schema_tables
+        schema_ex_col = {}
+        if cfg.CONF.ovsdb.schema_exclude_columns:
+            for c in cfg.CONF.ovsdb.schema_exclude_columns:
+                tbl, col = c.split('.')
+                if tbl in schema_ex_col:
+                    schema_ex_col[tbl].append(col)
+                else:
+                    schema_ex_col[tbl] = [col]
+
+        app = client.RemoteOvsdb.factory(sock, client_address,
+                                         probe_interval=self._probe_interval,
+                                         min_backoff=self._min_backoff,
+                                         max_backoff=self._max_backoff,
+                                         schema_tables=schema_tables,
+                                         schema_exclude_columns=schema_ex_col)
 
         if app:
             self._clients[app.name] = app
@@ -92,7 +150,11 @@ class OVSDB(app_manager.RyuApp):
             self.send_event_to_observers(ev)
 
         else:
-            sock.shutdown(socket.SHUT_RDWR)
+            try:
+                sock.shutdown(socket.SHUT_RDWR)
+            except:
+                pass
+
             sock.close()
 
     def start(self):
@@ -118,9 +180,19 @@ class OVSDB(app_manager.RyuApp):
         return t
 
     def stop(self):
-        for client in self._clients.values():
-            client.stop()
+        # NOTE(jkoelker) Attempt to gracefully stop the accept loop
+        self.is_active = False
 
+        # NOTE(jkoelker) Forceably kill the loop and clear the main_thread
+        if self.main_thread:
+            hub.kill(self.main_thread)
+            self.main_thread = None
+
+        # NOTE(jkoelker) Stop all the clients
+        for c in self._clients.values():
+            c.stop()
+
+        # NOTE(jkoelker) super will only take care of the event and joining now
         super(OVSDB, self).stop()
 
     @handler.set_ev_cls(event.EventModifyRequest)
@@ -141,6 +213,16 @@ class OVSDB(app_manager.RyuApp):
     @handler.set_ev_cls(event.EventReadRequest)
     def read_request_handler(self, ev):
         system_id = ev.system_id
+
+        if system_id is None:
+            def done(gt, *args, **kwargs):
+                if gt in self.threads:
+                    self.threads.remove(gt)
+
+            thread = hub.spawn(self._bulk_read_handler, ev)
+            self.threads.append(thread)
+            return thread.link(done)
+
         client_name = client.RemoteOvsdb.instance_name(system_id)
         remote = self._clients.get(client_name)
 
