@@ -14,15 +14,37 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from __future__ import print_function
+
+import logging
+import os
+import sys
+
 import unittest
 from nose.tools import eq_
 from nose.tools import ok_
 
+from ryu.utils import binary_str
+from ryu.lib import pcaplib
+from ryu.lib.packet import packet
 from ryu.lib.packet import bgp
 from ryu.lib.packet import afi
 from ryu.lib.packet import safi
 
 
+LOG = logging.getLogger(__name__)
+
+BGP4_PACKET_DATA_DIR = os.path.join(
+    os.path.dirname(sys.modules[__name__].__file__), '../../packet_data/bgp4/')
+
+PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT = (
+    bgp.BGPPathAttributePmsiTunnel.TYPE_NO_TUNNEL_INFORMATION_PRESENT
+)
+PMSI_TYPE_INGRESS_REPLICATION = (
+    bgp.BGPPathAttributePmsiTunnel.TYPE_INGRESS_REPLICATION
+)
+
+
 class Test_bgp(unittest.TestCase):
     """ Test case for ryu.lib.packet.bgp
     """
@@ -36,7 +58,7 @@ class Test_bgp(unittest.TestCase):
     def test_open1(self):
         msg = bgp.BGPOpen(my_as=30000, bgp_identifier='192.0.2.1')
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         eq_(len(msg), 29)
         eq_(rest, b'')
@@ -58,7 +80,7 @@ class Test_bgp(unittest.TestCase):
         msg = bgp.BGPOpen(my_as=30000, bgp_identifier='192.0.2.2',
                           opt_param=opt_param)
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         ok_(len(msg) > 29)
         eq_(rest, b'')
@@ -66,7 +88,7 @@ class Test_bgp(unittest.TestCase):
     def test_update1(self):
         msg = bgp.BGPUpdate()
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         eq_(len(msg), 23)
         eq_(rest, b'')
@@ -105,12 +127,20 @@ class Test_bgp(unittest.TestCase):
             bgp.BGPIPv4AddressSpecificExtendedCommunity(
                 subtype=3, ipv4_address='192.0.2.1',
                 local_administrator=65432),
-            bgp.BGPOpaqueExtendedCommunity(opaque=b'abcdefg'),
+            bgp.BGPOpaqueExtendedCommunity(subtype=13, opaque=b'abcdef'),
+            bgp.BGPEncapsulationExtendedCommunity(
+                subtype=0x0c, tunnel_type=10),
+            bgp.BGPEvpnMacMobilityExtendedCommunity(
+                subtype=0, flags=0xff, sequence_number=0x11223344),
+            bgp.BGPEvpnEsiLabelExtendedCommunity(
+                subtype=1, flags=0xff, esi_label=0x112233),
+            bgp.BGPEvpnEsImportRTExtendedCommunity(
+                subtype=2, es_import="aa:bb:cc:dd:ee:ff"),
             bgp.BGPUnknownExtendedCommunity(type_=99, value=b'abcdefg'),
         ]
         path_attributes = [
             bgp.BGPPathAttributeOrigin(value=1),
-            bgp.BGPPathAttributeAsPath(value=[[1000], set([1001, 1002]),
+            bgp.BGPPathAttributeAsPath(value=[[1000], {1001, 1002},
                                               [1003, 1004]]),
             bgp.BGPPathAttributeNextHop(value='192.0.2.199'),
             bgp.BGPPathAttributeMultiExitDisc(value=2000000000),
@@ -122,7 +152,32 @@ class Test_bgp(unittest.TestCase):
             bgp.BGPPathAttributeOriginatorId(value='10.1.1.1'),
             bgp.BGPPathAttributeClusterList(value=['1.1.1.1', '2.2.2.2']),
             bgp.BGPPathAttributeExtendedCommunities(communities=ecommunities),
-            bgp.BGPPathAttributeAs4Path(value=[[1000000], set([1000001, 1002]),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
+                label=b'\xFF\xFF\xFF'),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
+                tunnel_id=None),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
+                mpls_label=0xfffff,
+                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
+                    tunnel_endpoint_ip="1.1.1.1")),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
+                vni=0xffffff,
+                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
+                    tunnel_endpoint_ip="aa:bb:cc::dd:ee:ff")),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=2,
+                label=b'\xFF\xFF\xFF',
+                tunnel_id=bgp.PmsiTunnelIdUnknown(value=b'test')),
+            bgp.BGPPathAttributeAs4Path(value=[[1000000], {1000001, 1002},
                                                [1003, 1000004]]),
             bgp.BGPPathAttributeAs4Aggregator(as_number=100040000,
                                               addr='192.0.2.99'),
@@ -144,7 +199,7 @@ class Test_bgp(unittest.TestCase):
                             path_attributes=path_attributes,
                             nlri=nlri)
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         ok_(len(msg) > 23)
         eq_(rest, b'')
@@ -152,7 +207,7 @@ class Test_bgp(unittest.TestCase):
     def test_keepalive(self):
         msg = bgp.BGPKeepAlive()
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         eq_(len(msg), 19)
         eq_(rest, b'')
@@ -161,7 +216,7 @@ class Test_bgp(unittest.TestCase):
         data = b'hoge'
         msg = bgp.BGPNotification(error_code=1, error_subcode=2, data=data)
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         eq_(len(msg), 21 + len(data))
         eq_(rest, b'')
@@ -169,7 +224,7 @@ class Test_bgp(unittest.TestCase):
     def test_route_refresh(self):
         msg = bgp.BGPRouteRefresh(afi=afi.IP, safi=safi.MPLS_VPN)
         binmsg = msg.serialize()
-        msg2, rest = bgp.BGPMessage.parser(binmsg)
+        msg2, _, rest = bgp.BGPMessage.parser(binmsg)
         eq_(str(msg), str(msg2))
         eq_(len(msg), 23)
         eq_(rest, b'')
@@ -191,21 +246,36 @@ class Test_bgp(unittest.TestCase):
     def test_parser(self):
         files = [
             'bgp4-open',
-            # commented out because
-            # 1. we don't support 32 bit AS numbers in AS_PATH
-            # 2. quagga always uses EXTENDED for AS_PATH
-            # 'bgp4-update',
+            'bgp4-update',
+            'bgp4-update_ipv6',
+            'bgp4-update_vpnv6',
             'bgp4-keepalive',
+            'evpn_esi_arbitrary',
+            'evpn_esi_lacp',
+            'evpn_esi_l2_bridge',
+            'evpn_esi_mac_base',
+            'evpn_esi_router_id',
+            'evpn_esi_as_based',
+            'evpn_nlri_eth_a-d',
+            'evpn_nlri_mac_ip_ad',
+            'evpn_nlri_inc_multi_eth_tag',
+            'evpn_nlri_eth_seg',
+            'evpn_nlri_ip_prefix',
         ]
-        dir = '../packet_data/bgp4/'
 
         for f in files:
-            print('testing %s' % f)
-            binmsg = open(dir + f, 'rb').read()
-            msg, rest = bgp.BGPMessage.parser(binmsg)
-            binmsg2 = msg.serialize()
-            eq_(binmsg, binmsg2)
-            eq_(rest, b'')
+            LOG.debug('*** testing %s ...', f)
+            for _, buf in pcaplib.Reader(
+                    open(BGP4_PACKET_DATA_DIR + f + '.pcap', 'rb')):
+                # Checks if BGP message can be parsed as expected.
+                pkt = packet.Packet(buf)
+                ok_(isinstance(pkt.protocols[-1], bgp.BGPMessage),
+                    'Failed to parse BGP message: %s' % pkt)
+
+                # Checks if BGP message can be serialized as expected.
+                pkt.serialize()
+                eq_(buf, pkt.data,
+                    "b'%s' != b'%s'" % (binary_str(buf), binary_str(pkt.data)))
 
     def test_json1(self):
         opt_param = [bgp.BGPOptParamCapabilityUnknown(cap_code=200,
@@ -253,12 +323,20 @@ class Test_bgp(unittest.TestCase):
             bgp.BGPIPv4AddressSpecificExtendedCommunity(
                 subtype=3, ipv4_address='192.0.2.1',
                 local_administrator=65432),
-            bgp.BGPOpaqueExtendedCommunity(opaque=b'abcdefg'),
+            bgp.BGPOpaqueExtendedCommunity(subtype=13, opaque=b'abcdef'),
+            bgp.BGPEncapsulationExtendedCommunity(
+                subtype=0x0c, tunnel_type=10),
+            bgp.BGPEvpnMacMobilityExtendedCommunity(
+                subtype=0, flags=0xff, sequence_number=0x11223344),
+            bgp.BGPEvpnEsiLabelExtendedCommunity(
+                subtype=1, flags=0xff, esi_label=0x112233),
+            bgp.BGPEvpnEsImportRTExtendedCommunity(
+                subtype=2, es_import="aa:bb:cc:dd:ee:ff"),
             bgp.BGPUnknownExtendedCommunity(type_=99, value=b'abcdefg'),
         ]
         path_attributes = [
             bgp.BGPPathAttributeOrigin(value=1),
-            bgp.BGPPathAttributeAsPath(value=[[1000], set([1001, 1002]),
+            bgp.BGPPathAttributeAsPath(value=[[1000], {1001, 1002},
                                               [1003, 1004]]),
             bgp.BGPPathAttributeNextHop(value='192.0.2.199'),
             bgp.BGPPathAttributeMultiExitDisc(value=2000000000),
@@ -268,7 +346,32 @@ class Test_bgp(unittest.TestCase):
                                            addr='192.0.2.99'),
             bgp.BGPPathAttributeCommunities(communities=communities),
             bgp.BGPPathAttributeExtendedCommunities(communities=ecommunities),
-            bgp.BGPPathAttributeAs4Path(value=[[1000000], set([1000001, 1002]),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
+                label=b'\xFF\xFF\xFF'),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_NO_TUNNEL_INFORMATION_PRESENT,
+                tunnel_id=None),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
+                mpls_label=0xfffff,
+                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
+                    tunnel_endpoint_ip="1.1.1.1")),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=PMSI_TYPE_INGRESS_REPLICATION,
+                vni=0xffffff,
+                tunnel_id=bgp.PmsiTunnelIdIngressReplication(
+                    tunnel_endpoint_ip="aa:bb:cc::dd:ee:ff")),
+            bgp.BGPPathAttributePmsiTunnel(
+                pmsi_flags=1,
+                tunnel_type=2,
+                label=b'\xFF\xFF\xFF',
+                tunnel_id=bgp.PmsiTunnelIdUnknown(value=b'test')),
+            bgp.BGPPathAttributeAs4Path(value=[[1000000], {1000001, 1002},
                                                [1003, 1000004]]),
             bgp.BGPPathAttributeAs4Aggregator(as_number=100040000,
                                               addr='192.0.2.99'),
