@@ -14,10 +14,11 @@
 # limitations under the License.
 
 
-import unittest
 import inspect
 import logging
+import six
 import struct
+import unittest
 
 from nose.tools import eq_
 from ryu.lib.packet import icmp
@@ -50,18 +51,18 @@ class Test_icmp(unittest.TestCase):
 
         self.buf = bytearray(struct.pack(
             icmp.icmp._PACK_STR, self.type_, self.code, self.csum))
-        self.csum_calc = packet_utils.checksum(str(self.buf))
+        self.csum_calc = packet_utils.checksum(self.buf)
         struct.pack_into('!H', self.buf, 2, self.csum_calc)
 
     def setUp_with_echo(self):
         self.echo_id = 13379
         self.echo_seq = 1
-        self.echo_data = '\x30\x0e\x09\x00\x00\x00\x00\x00' \
-            + '\x10\x11\x12\x13\x14\x15\x16\x17' \
-            + '\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f' \
-            + '\x20\x21\x22\x23\x24\x25\x26\x27' \
-            + '\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f' \
-            + '\x30\x31\x32\x33\x34\x35\x36\x37'
+        self.echo_data = b'\x30\x0e\x09\x00\x00\x00\x00\x00' \
+            + b'\x10\x11\x12\x13\x14\x15\x16\x17' \
+            + b'\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f' \
+            + b'\x20\x21\x22\x23\x24\x25\x26\x27' \
+            + b'\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f' \
+            + b'\x30\x31\x32\x33\x34\x35\x36\x37'
         self.data = icmp.echo(
             id_=self.echo_id, seq=self.echo_seq, data=self.echo_data)
 
@@ -69,15 +70,15 @@ class Test_icmp(unittest.TestCase):
         self.code = 0
         self.ic = icmp.icmp(self.type_, self.code, self.csum, self.data)
 
-        self.buf = struct.pack(
-            icmp.icmp._PACK_STR, self.type_, self.code, self.csum)
+        self.buf = bytearray(struct.pack(
+            icmp.icmp._PACK_STR, self.type_, self.code, self.csum))
         self.buf += self.data.serialize()
-        self.csum_calc = packet_utils.checksum(str(self.buf))
+        self.csum_calc = packet_utils.checksum(self.buf)
         struct.pack_into('!H', self.buf, 2, self.csum_calc)
 
     def setUp_with_dest_unreach(self):
         self.unreach_mtu = 10
-        self.unreach_data = 'abc'
+        self.unreach_data = b'abc'
         self.unreach_data_len = len(self.unreach_data)
         self.data = icmp.dest_unreach(
             data_len=self.unreach_data_len, mtu=self.unreach_mtu,
@@ -87,14 +88,14 @@ class Test_icmp(unittest.TestCase):
         self.code = icmp.ICMP_HOST_UNREACH_CODE
         self.ic = icmp.icmp(self.type_, self.code, self.csum, self.data)
 
-        self.buf = struct.pack(
-            icmp.icmp._PACK_STR, self.type_, self.code, self.csum)
+        self.buf = bytearray(struct.pack(
+            icmp.icmp._PACK_STR, self.type_, self.code, self.csum))
         self.buf += self.data.serialize()
-        self.csum_calc = packet_utils.checksum(str(self.buf))
+        self.csum_calc = packet_utils.checksum(self.buf)
         struct.pack_into('!H', self.buf, 2, self.csum_calc)
 
     def setUp_with_TimeExceeded(self):
-        self.te_data = 'abc'
+        self.te_data = b'abc'
         self.te_data_len = len(self.te_data)
         self.data = icmp.TimeExceeded(
             data_len=self.te_data_len, data=self.te_data)
@@ -103,10 +104,10 @@ class Test_icmp(unittest.TestCase):
         self.code = 0
         self.ic = icmp.icmp(self.type_, self.code, self.csum, self.data)
 
-        self.buf = struct.pack(
-            icmp.icmp._PACK_STR, self.type_, self.code, self.csum)
+        self.buf = bytearray(struct.pack(
+            icmp.icmp._PACK_STR, self.type_, self.code, self.csum))
         self.buf += self.data.serialize()
-        self.csum_calc = packet_utils.checksum(str(self.buf))
+        self.csum_calc = packet_utils.checksum(self.buf)
         struct.pack_into('!H', self.buf, 2, self.csum_calc)
 
     def test_init(self):
@@ -128,7 +129,7 @@ class Test_icmp(unittest.TestCase):
         self.test_init()
 
     def test_parser(self):
-        _res = icmp.icmp.parser(str(self.buf))
+        _res = icmp.icmp.parser(six.binary_type(self.buf))
         if type(_res) is tuple:
             res = _res[0]
         else:
@@ -156,7 +157,7 @@ class Test_icmp(unittest.TestCase):
         prev = None
         buf = self.ic.serialize(data, prev)
 
-        res = struct.unpack_from(icmp.icmp._PACK_STR, str(buf))
+        res = struct.unpack_from(icmp.icmp._PACK_STR, six.binary_type(buf))
 
         eq_(self.type_, res[0])
         eq_(self.code, res[1])
@@ -169,7 +170,7 @@ class Test_icmp(unittest.TestCase):
         data = bytearray()
         prev = None
         buf = self.ic.serialize(data, prev)
-        echo = icmp.echo.parser(str(buf), icmp.icmp._MIN_LEN)
+        echo = icmp.echo.parser(six.binary_type(buf), icmp.icmp._MIN_LEN)
         eq_(repr(self.data), repr(echo))
 
     def test_serialize_with_dest_unreach(self):
@@ -179,7 +180,7 @@ class Test_icmp(unittest.TestCase):
         data = bytearray()
         prev = None
         buf = self.ic.serialize(data, prev)
-        unreach = icmp.dest_unreach.parser(str(buf), icmp.icmp._MIN_LEN)
+        unreach = icmp.dest_unreach.parser(six.binary_type(buf), icmp.icmp._MIN_LEN)
         eq_(repr(self.data), repr(unreach))
 
     def test_serialize_with_TimeExceeded(self):
@@ -189,7 +190,7 @@ class Test_icmp(unittest.TestCase):
         data = bytearray()
         prev = None
         buf = self.ic.serialize(data, prev)
-        te = icmp.TimeExceeded.parser(str(buf), icmp.icmp._MIN_LEN)
+        te = icmp.TimeExceeded.parser(six.binary_type(buf), icmp.icmp._MIN_LEN)
         eq_(repr(self.data), repr(te))
 
     def test_to_string(self):
@@ -220,20 +221,20 @@ class Test_icmp(unittest.TestCase):
     def test_default_args(self):
         ic = icmp.icmp()
         buf = ic.serialize(bytearray(), None)
-        res = struct.unpack(icmp.icmp._PACK_STR, str(buf[:4]))
+        res = struct.unpack(icmp.icmp._PACK_STR, six.binary_type(buf[:4]))
 
         eq_(res[0], 8)
         eq_(res[1], 0)
-        eq_(buf[4:], '\x00\x00\x00\x00')
+        eq_(buf[4:], b'\x00\x00\x00\x00')
 
         # with data
         ic = icmp.icmp(type_=icmp.ICMP_DEST_UNREACH, data=icmp.dest_unreach())
         buf = ic.serialize(bytearray(), None)
-        res = struct.unpack(icmp.icmp._PACK_STR, str(buf[:4]))
+        res = struct.unpack(icmp.icmp._PACK_STR, six.binary_type(buf[:4]))
 
         eq_(res[0], 3)
         eq_(res[1], 0)
-        eq_(buf[4:], '\x00\x00\x00\x00')
+        eq_(buf[4:], b'\x00\x00\x00\x00')
 
     def test_json(self):
         jsondict = self.ic.to_jsondict()
@@ -258,12 +259,12 @@ class Test_echo(unittest.TestCase):
     def setUp(self):
         self.id_ = 13379
         self.seq = 1
-        self.data = '\x30\x0e\x09\x00\x00\x00\x00\x00' \
-            + '\x10\x11\x12\x13\x14\x15\x16\x17' \
-            + '\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f' \
-            + '\x20\x21\x22\x23\x24\x25\x26\x27' \
-            + '\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f' \
-            + '\x30\x31\x32\x33\x34\x35\x36\x37'
+        self.data = b'\x30\x0e\x09\x00\x00\x00\x00\x00' \
+            + b'\x10\x11\x12\x13\x14\x15\x16\x17' \
+            + b'\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f' \
+            + b'\x20\x21\x22\x23\x24\x25\x26\x27' \
+            + b'\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f' \
+            + b'\x30\x31\x32\x33\x34\x35\x36\x37'
         self.echo = icmp.echo(
             self.id_, self.seq, self.data)
         self.buf = struct.pack('!HH', self.id_, self.seq)
@@ -286,7 +287,7 @@ class Test_echo(unittest.TestCase):
 
     def test_serialize(self):
         buf = self.echo.serialize()
-        res = struct.unpack_from('!HH', str(buf))
+        res = struct.unpack_from('!HH', six.binary_type(buf))
         eq_(self.id_, res[0])
         eq_(self.seq, res[1])
         eq_(self.data, buf[struct.calcsize('!HH'):])
@@ -294,7 +295,7 @@ class Test_echo(unittest.TestCase):
     def test_default_args(self):
         ec = icmp.echo()
         buf = ec.serialize()
-        res = struct.unpack(icmp.echo._PACK_STR, str(buf))
+        res = struct.unpack(icmp.echo._PACK_STR, six.binary_type(buf))
 
         eq_(res[0], 0)
         eq_(res[1], 0)
@@ -304,7 +305,7 @@ class Test_dest_unreach(unittest.TestCase):
 
     def setUp(self):
         self.mtu = 10
-        self.data = 'abc'
+        self.data = b'abc'
         self.data_len = len(self.data)
         self.dest_unreach = icmp.dest_unreach(
             data_len=self.data_len, mtu=self.mtu, data=self.data)
@@ -328,7 +329,7 @@ class Test_dest_unreach(unittest.TestCase):
 
     def test_serialize(self):
         buf = self.dest_unreach.serialize()
-        res = struct.unpack_from('!xBH', str(buf))
+        res = struct.unpack_from('!xBH', six.binary_type(buf))
         eq_(self.data_len, res[0])
         eq_(self.mtu, res[1])
         eq_(self.data, buf[struct.calcsize('!xBH'):])
@@ -336,7 +337,7 @@ class Test_dest_unreach(unittest.TestCase):
     def test_default_args(self):
         du = icmp.dest_unreach()
         buf = du.serialize()
-        res = struct.unpack(icmp.dest_unreach._PACK_STR, str(buf))
+        res = struct.unpack(icmp.dest_unreach._PACK_STR, six.binary_type(buf))
 
         eq_(res[0], 0)
         eq_(res[1], 0)
@@ -345,7 +346,7 @@ class Test_dest_unreach(unittest.TestCase):
 class Test_TimeExceeded(unittest.TestCase):
 
     def setUp(self):
-        self.data = 'abc'
+        self.data = b'abc'
         self.data_len = len(self.data)
         self.te = icmp.TimeExceeded(
             data_len=self.data_len, data=self.data)
@@ -367,13 +368,13 @@ class Test_TimeExceeded(unittest.TestCase):
 
     def test_serialize(self):
         buf = self.te.serialize()
-        res = struct.unpack_from('!xBxx', str(buf))
+        res = struct.unpack_from('!xBxx', six.binary_type(buf))
         eq_(self.data_len, res[0])
         eq_(self.data, buf[struct.calcsize('!xBxx'):])
 
     def test_default_args(self):
         te = icmp.TimeExceeded()
         buf = te.serialize()
-        res = struct.unpack(icmp.TimeExceeded._PACK_STR, str(buf))
+        res = struct.unpack(icmp.TimeExceeded._PACK_STR, six.binary_type(buf))
 
         eq_(res[0], 0)
